<?php
/**
 * La Strada Loader Class
 */
class La_Strada_Loader
{
    var $plugin_url;
    var $key;
    var $lsl_config_data;
    
    function lsloader(){
        $this->__construct();
    }
    
    /**
     * Initiate  plugin
     */
    function __construct(){
        $this->plugin_url = trailingslashit(get_bloginfo('wpurl')) . PLUGINDIR . '/' . dirname(plugin_basename(__FILE__));
        $this->key = 'lsloader';
        $this->lsl_set_config_data();
    }

    /**
     * Downloading and activate theme
     *
     * @param string $name 		Theme name
     * @param string $package 	Download URL of the theme
     */
    function lsl_theme_handle_download($name, $package) {
    	global $wp_version;

    	include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';  

    	$upgrader = new Theme_Upgrader(); 
    	$upgrader->install($package);
    	if ( ! $upgrader->theme_info() ){
        		echo '<a href="' . wp_nonce_url('themes.php?action=activate&amp;template=' . $name . '&amp;stylesheet=' . $name) . '" title="' . esc_attr__('Activate this Theme') . '" target="_parent">' . __('Activate Theme') . '</a>';
        } else {
        	update_option('template', $name);	
        	update_option('stylesheet', $name);	
        	_e('<b class="mpi_act">Theme activated successfully.</b><br/>','mpi');
        }
    }

	/**
	 * Downloading and activate an array of given plugins
	 *
	 * @param string $plugin_name 	Plugin name
	 * @param string $package 		Plugin download URL
	 */
    function mpi_plugin_handle_download($plugin_name, $package)
	{
        global $wp_version;

        if(version_compare($wp_version, '3.0', '<')){
            include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';  
            $upgrader = new Plugin_Upgrader(); 
            $upgrader->install($package);
            
        	if ($upgrader->plugin_info()){
        		echo '<a href="' . wp_nonce_url('plugins.php?action=activate&amp;plugin=' . $upgrader->plugin_info(), 'activate-plugin_' . $plugin_file) . '" title="' . esc_attr__('Activate this plugin') . '" target="_parent">' . __('Activate Plugin') . '</a>';
        	}
        }			
        else{
			include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
			$upgrader = new Plugin_Upgrader( new Plugin_Installer_Skin( compact('type', 'title', 'nonce', 'url') ) ); 
			$res=$upgrader->install($package);  

			if (! $upgrader->plugin_info() ){
				echo $res;
			}
			else {
				$mpiplugins = get_option('active_plugins');
				if($mpiplugins){
					$puginsToActiv = array($upgrader->plugin_info());
					foreach ($puginsToActiv as $mpiplugin){
						if (!in_array($mpiplugin, $mpiplugins)) {
							 array_push($mpiplugins,$mpiplugin);
							 update_option('active_plugins',$mpiplugins);
						}
					}
				}
				_e('<b class="mpi_act">Plugin activated successfully.</b><br/>','mpi');
			}
        }
    }
	
    /**
	 * Call Wordpress remote_post() to get plugin/theme information from Wordpress API
	 *
	 * @param string $url 	URL to call
	 * @param string $data 	POST arguments/parameters
	 */
    function post_request($url, $data) {
    	global $wp_version;

    	$data = http_build_query($data);
	    $url = parse_url($url);

    	$host = $url['host'];
	    $path = $url['path'];

		if ( function_exists( 'wp_remote_post' ) ) {
			$http_args = array(
				'body'			=> $data,
				'method'		=> 'POST',
				'headers'		=> array(
					'Content-Type'	=> 'application/x-www-form-urlencoded; charset=' . get_option( 'blog_charset' ),
					'Host'			=> $host,
					'User-Agent'	=> 'User-Agent: WordPress/' . $wp_version . '; ' . get_bloginfo('url')
				),
				'httpversion'	=> '1.0',
				'timeout'		=> 15
			);
			$url = "http://{$host}{$path}";
			$response = wp_remote_post( $url, $http_args );

			if ( is_wp_error( $response ) ) {
				$error_message = $response->get_error_message();
				return array('status' => 'err', 'error' => "$error_message");
			} else {
				return array(
			        'status' => 'ok',
			        'content' => $response['body']
			    );
			}
		} else {
			die("Function wp_remote_post() doesn\'t exist.");
		}
	}

	/**
	 * Initial theme installation by calling wordpress-API and lsl_theme_handle_download()
	 *
	 * @param string $theme_name Name/Slug of the theme
	 */
    function lsl_get_theme($theme_name) {
        echo "<p><b>Theme Installation</b></p>";
        $theme_name = trim($theme_name);
        
        if ( end(explode(".", $theme_name)) == 'zip' ) {
        	//$this->package_handle_download("theme", $theme_name, $theme_name);
        	echo "Installing theme from ZIP not yet implemented";
        }
        else {
	    	$url = 'http://api.wordpress.org/themes/info/1.0/';
	       	$request = new StdClass();
	        $request->slug = stripslashes($theme_name);
	       	$data = array('action' => 'theme_information', 'request' => serialize($request));           	
			$response = $this->post_request($url, $data);

			$r = @unserialize($response["content"]);
    		if ( $r != NULL ) {
    			$this->_mpiflush("<p class=\"found\">Found <strong>" . $theme_name . "</strong>. Processing installation...</strong></p>");
				$this->lsl_theme_handle_download($theme_name, $r->download_link);
    		} else {
    			echo '<p class="not-found"><strong>' . $theme_name . '</strong> not found. Try <a href="http://google.com/search?q=' . $theme_name . ' +wordpress">manual</a> install.</p>';
    		}
		}
        
    }
 
    /**
     * Initial the plugin installation  by calling wordpress-API and mpi_theme_plugin_download()
     *
     * @param array $plugins_array 	Name/Slug-Array of plugins
     */
    function mpi_get_packages($plugins_arr)
	{       
        $url = 'http://api.wordpress.org/plugins/info/1.0/';
        foreach ($plugins_arr as $val){
            $val = trim($val);
            
            if (end(explode(".", $val)) == 'zip'){
            	//handle zip-file
               $this->mpi_plugin_handle_download("temp",$val);
            }
            else {
                $request = new StdClass();
        		$request->slug = $val;
        		$data = array('action' => 'plugin_information', 'request' => serialize($request) );
        		$response = $this->post_request($url, $data);

        		$r = @unserialize($response["content"]);
        		if ( $r != NULL ) {
        			$this->_mpiflush("<p class=\"found\">Found <strong>" . $val . "</strong> ($r->slug, version $r->version). Processing installation...</strong></p>");
					$this->mpi_plugin_handle_download($r->slug, $r->download_link);
        		} else {
        			echo '<p class="not-found"><strong>' . $val . '</strong> not found. Try <a href="http://google.com/search?q=' . $val . ' +wordpress">manual</a> install.</p>';
        		}
            }
        }        
    }

    function _mpiflush($s){
        echo $s;
        flush();
    }

    /**
     * Tests if upload direcory is writeable
     *
     * @return boolean true or false if writealble or not
     */
	function mpi_app_DirTesting(){
		if(!is_dir(mpiUPLOADDIR_PATH.'/mpi_testing')){ 
			if(@mkdir(mpiUPLOADDIR_PATH.'/mpi_testing', 0777)){
				@rmdir(mpiUPLOADDIR_PATH.'/mpi_testing');
				return true;
			}
			else
			return false;
		}
	}	

	/**
	 * Retrieve current configuration file and set corresponding class-variable
	 */
	function lsl_set_config_data() {
		$confId = isset( $_GET['confID'] ) ? $_GET['confID'] : 0;
		$config_files = $this->lsl_get_config_files();
		$my_config = mpiPLUGIN_PATH . 'config/' . $config_files[$confId]['file'];

		if ( ! is_file($my_config))
			die('Couldn\'t find config-file "'  . $config_files[$confId]['file'] . '"');

		include($my_config);
		$this->lsl_config_data = array(			
			'CONFIG_NAME'	=> $lsl_CONFIG_NAME,							
			'options' 		=> array (
								// !!! options-array-key must be the update_option()-key !!!
								'show_on_front' => $show_on_front,
									),
			'permalink' 	=> $permalink,
			'categories' 	=> $categories,
			'pages'			=> $pages,
			'menus'			=> $menus,
			'text_widgets'	=> $text_widgets,
			'sidebars' 		=> $sidebars,
			'theme'			=> $theme,
			'plugins' 		=> $plugins,
		);
	}

	/**
	 * Get all available configuration-files
	 *
	 * @return array Array with founded configurations (name and file)
	 */
	function lsl_get_config_files() {		
		$configDir = mpiPLUGIN_PATH . 'config/';
		$configDir_files = scandir($configDir);
		$config_files = array();
		foreach ($configDir_files as $file) {
			if (is_file($configDir. $file)) {
				include($configDir . $file);
				array_push($config_files, array('name' => $lsl_CONFIG_NAME, 'file' => $file));
			}
		}
		return $config_files;
	}

	function lsl_get_config_data()
	{
		return $this->lsl_config_data;
	}

	/**
	 * Initial the plugin installation by getting POST list and calling mpi_get_packages()
	 */
	function mpi_app_wpInstall(){
		check_admin_referer($this->key);
		_e('<div class="mpi_h3">Plugin installation process:</div>','mpi');
		$plugin_install = !isset($_POST['mpi_wplists']) ? '' : $_POST['mpi_wplists'];
		if ($plugin_install != '') {
			$plugin_install = str_replace(array("\r\r\r", "\r\r", "\r\n", "\n\r", "\n\n\n", "\n\n"), "\n", $plugin_install);
			$options = explode("\n", $plugin_install);
			$this->mpi_get_packages($options);
		}
	}

	/**
	 * Start main installation process with retrieved data from POST.
	 * Sets worpdress options/settings, adds pages, categorys, menues, widgets
	 * Calls theme and plugin installation process
	 */
	function lsl_install() {
		check_admin_referer($this->key);
		$lsl_config_data = $this->stripslashes_deep($_POST['lsl_config_data']);

		//set standard array-keys from config-file if empty
		foreach (array_keys($this->lsl_get_config_data()) as $key) {
			if ( ! isset($lsl_config_data[$key]) )
				$lsl_config_data[$key] = array();
		}
		//echo "<pre>"; var_dump($_POST); echo "</pre><hr>";		

		/* CONFIGURATION */	
		echo "<div class='mpi_h3'>La Strada installer:</div>";
		//udpate DB		
		foreach ( $lsl_config_data['options'] as $key => $value ) {
			if ( ! empty($value) ) {
				echo "<p>- update Wordpress-Option $key to $value</p>";
				update_option($key, $value);
			}
		}

		//set permalink-structure
		if ( ! empty($lsl_config_data['permalink']) ) {
			echo "<p>- set parmalink-structure to '" . $lsl_config_data['permalink'] . "'</p>";
			add_action( 'init', function() {
					global $wp_rewrite;
					$wp_rewrite->set_permalink_structure($lsl_config_data['permalink']);
				} );
		}
	
		//add categories
		if ( ! empty($lsl_config_data['categories']) ) {
			$lsl_config_data['categories'] = explode(',', $lsl_config_data['categories']);
			foreach ( $lsl_config_data['categories'] as $category ) {
				echo "<p>- create category '" . $category . "'</p>";
				wp_create_category($category);
			}
		}	
			
		//add pages
		foreach ( $lsl_config_data['pages'] as $key => $value ) {
			echo "<p>- create page '" . $value['title'] . "'</p>";
			$page = array('post_title' => $value['title'],
						  'post_content' => $value['content'],
						  'post_status' => 'publish',
						  'post_type' => 'page');
			wp_insert_post($page);
		}		

		/* DESIGN */
		//download + install theme
		if ( ! empty ($lsl_config_data['theme']) )
			$this->lsl_get_theme( $lsl_config_data['theme'] );
	
		//create mainmenu + items
		if ( ( ! empty($lsl_config_data['pages']) || ! empty($lsl_config_data['menus']) )  
			 && ! wp_get_nav_menu_object( 'Hauptmenü' )
			) {
			$menu_id = wp_create_nav_menu( 'Hauptmenü' );
			foreach ( $lsl_config_data['pages'] as $key => $value ) {
				wp_update_nav_menu_item($menu_id, 0, array('menu-item-title' => $value['title'],
                                           'menu-item-object' => 'page',
                                           'menu-item-object-id' => get_page_by_title($value['title'])->ID,
                                           'menu-item-type' => 'post_type',
                                           'menu-item-status' => 'publish'));
			}
		    //add aditional menu-items
			foreach ( $lsl_config_data['menus'] as $key => $value ) {
				wp_update_nav_menu_item($menu_id, 0, array('menu-item-title' => $value['title'],
                                           'menu-item-url' => $value['url'],
                                           'menu-item-type' => 'custom',
                                           'menu-item-status' => 'publish'));
			}
			if( ! has_nav_menu( 'primary' ) ){
				echo "<p>- create main-menu at primary</p>";
		        $locations = get_theme_mod('nav_menu_locations');
		        $locations['primary'] = $menu_id;
		        set_theme_mod( 'nav_menu_locations', $locations );
		    }
		}
		
		//create text-widgets
		$widget_array = array();
		foreach ($lsl_config_data['text_widgets'] as $key => $value) {
			$widget_array[$key+2] = array('title' => $value['title'], 'text' => $value['content'], 'filter' => '');
		}
		$widget_array['_multiwidget'] = '1';
		update_option("widget_text", $widget_array);

		//create sidebar-configs
		$sidebar_array = array();
		foreach ($lsl_config_data['sidebars'] as $key => $value) {
			$sidebar_name = array_keys(wp_get_widget_defaults())[$key];
			//$value = preg_replace("/mein-text-widget-([0-9])/", 'text', $value);
			eval("\$sidebar_array[\$sidebar_name] = $value;");
		}
		$sidebar_array['wp_inactive_widgets'] = array();
		update_option( 'sidebars_widgets', $sidebar_array );
		
		/* PLUGINS */		
		// plugin-settings
		if ( ! empty($lsl_config_data['plugins']) )
			echo "<p><b>Plugin settings</b></p>";		
		
		foreach ($lsl_config_data['plugins'] as $plugin => $value) {
			if ( ! empty($value) ) {
				eval("\$value = $value;");
				echo "<p>- $plugin</p>";
				foreach ($value as $key => $option) {
					$p_option = print_r($option, true);
					update_option($key, $option);
				}
			}
		}

		//plugins download+installieren
		if ( ! empty ($_POST['mpi_wplists']) )
			$this->mpi_app_wpInstall();	
	}

	/**
	 * Removes backslashes from array-values rekursiv
	 */
	function stripslashes_deep($value) {
    	$value = is_array($value) ?
                array_map('stripslashes_deep', $value) :
                stripslashes($value);
    	return $value;
	}
} //end class
?>
